local FORCE_DEBUG = false;
local Debug = LE_SpecManager.Debug;

if LE_SpecManager:IsCompatableWOWVersion() then
	local moduleName = "EquipManager";
	local EquipManager = LE_SpecManager:NewModule(moduleName, "AceConsole-3.0", "AceEvent-3.0");
	LE_SpecManager.EquipManager = EquipManager;

	local equipSuccess = true;

	local function EquipmentSet_Equipped(specInfo)
		local _, setID, isEquipped = GetEquipmentSetInfoByName(specInfo.name);
		
		return (setID and isEquipped) or false;	
	end

	local function EquipmentSet_Exists(specInfo)
		local _, setID = GetEquipmentSetInfoByName(specInfo.name);

		return (setID ~= nil) or false;
	end

	function EquipManager:OnEnable()
		self:RegisterMessage("LE_SpecManager_Load", "Save");
		self:RegisterMessage("LE_SpecManager_SpecAdded", "Load");
		self:RegisterMessage("LE_SpecManager_SpecChange_Start", "Save");
		self:RegisterMessage("LE_SpecManager_SpecChange_Finish", "Load");
		self:RegisterMessage("LE_SpecManager_SpecRemoved", "Save");
		self:RegisterMessage("LE_SpecManager_LeftCombat", "Load");
	end

	function EquipManager:OnDisable()
		self:UnregisterMessage("LE_SpecManager_Load");
		self:UnregisterMessage("LE_SpecManager_SpecAdded");
		self:UnregisterMessage("LE_SpecManager_SpecChange_Start");
		self:UnregisterMessage("LE_SpecManager_SpecChange_Finish");
		self:UnregisterMessage("LE_SpecManager_SpecRemoved");
		self:UnregisterMessage("LE_SpecManager_LeftCombat");
	end


	function EquipManager:Load(event, specInfo)
		if specInfo and (event ~= "LE_SpecManager_LeftCombat" or (event == "LE_SpecManager_LeftCombat" and not equipSuccess)) then
			if EquipmentSet_Exists(specInfo) and not EquipmentSet_Equipped(specInfo) then
				equipSuccess = UseEquipmentSet(specInfo.name);
			elseif not EquipmentSet_Exists(specInfo) then
				EquipManager:Save(event, specInfo);
			end
		end
	end

	function EquipManager:Save(event, specInfo)
		if specInfo then
			if not EquipmentSet_Exists(specInfo) or (EquipmentSet_Exists(specInfo) and not EquipmentSet_Equipped(specInfo)) then
				SaveEquipmentSet(specInfo.name, string.gsub(string.upper(specInfo.icon), "INTERFACE\\ICONS\\", ""));
			end
		end
	end
end