local FORCE_DEBUG = false;
local Debug = LE_SpecManager.Debug;

local baseDataBroker = LibStub("LibDataBroker-1.1");
local addonIcon = LibStub("LibDBIcon-1.0");

if LE_SpecManager:IsCompatableWOWVersion() and baseDataBroker and addonIcon then
	local moduleName = "LDBManager";
	local LDBManager = LE_SpecManager:NewModule(moduleName, "AceConsole-3.0", "AceEvent-3.0");
	LE_SpecManager.LDBManager = LDBManager;

	local dataTextName = "LunaEclipse: Spec Manager";
	local lootSpecializationMenuName = "LunaEclipse_LootSpecialization_Menu";
	
	local reloadRequired;
	
	local iconTooltip = nil;
	local dataBroker = nil;

	local menuFrame = CreateFrame("Frame", lootSpecializationMenuName, UIParent, "UIDropDownMenuTemplate");
	local menuList = {
		{ text = SELECT_LOOT_SPECIALIZATION, isTitle = true, notCheckable = true },
		{ notCheckable = true, func = function() SetLootSpecialization(0) end },
		{ notCheckable = true },
		{ notCheckable = true },
		{ notCheckable = true },
		{ notCheckable = true },
	};
	
	local LDBDefaults = {
		icon = "Interface\\ICONS\\spell_nature_bloodlust",
		hide = false,
	};
	
	local clickTexture = {
		texture = "Interface\\TUTORIALFRAME\\UI-TUTORIAL-FRAME",
		height = "16", width = "12",
		xOffset = "0", yOffset = "-1",
		xDimension = "512",	yDimension = "512",
		leftClick = {
			left = "9",	right = "69", top = "228", bottom = "308",
		},
		rightClick = {
			left = "9",	right = "69", top = "330", bottom = "410",
		},
	};

	local activeString = string.join("", "|cff00FF00", ACTIVE_PETS, "|r");
	local inactiveString = string.join("", "|cffFF0000", FACTION_INACTIVE, "|r");	
	local leftButtonTextureString = string.join("", "|T", clickTexture.texture, ":", string.join(":", clickTexture.height, clickTexture.width, clickTexture.xOffset, clickTexture.yOffset, clickTexture.xDimension, clickTexture.yDimension, clickTexture.leftClick.left, clickTexture.leftClick.right, clickTexture.leftClick.top, clickTexture.leftClick.bottom), "|t");
	local rightButtonTextureString = string.join("", "|T", clickTexture.texture, ":", string.join(":", clickTexture.height, clickTexture.width, clickTexture.xOffset, clickTexture.yOffset, clickTexture.xDimension, clickTexture.yDimension, clickTexture.rightClick.left, clickTexture.rightClick.right, clickTexture.rightClick.top, clickTexture.rightClick.bottom), "|t");

	StaticPopupDialogs["LESM_CONFIRM_RELOAD_UI"] = {
		text = string.join("", "Please confirm you wish to Reload UI.", "\n\n", "This ensures that your profile currently loaded in memory is written to your hard disk!", "\n\n"),
		button1	= "Yes",
		button2	= "No",
		OnAccept = function(self) 
			ReloadUI(); 
		end,
		timeout	= 10,
		whileDead = true,
		hideOnEscape = true,
	};

	StaticPopupDialogs["LESM_TOGGLE_MINIMAP"] = {
		text = string.join("", "A Reload UI is required for the icon to become visible when using MiniMap icon manager addons.", "\n\n", "Do you wish to reload now?", "\n\n"),
		button1	= "Yes",
		button2	= "No",
		OnAccept = function(self) 
			ReloadUI(); 
		end,
		timeout	= 15,
		whileDead = true,
		hideOnEscape = true,
	};

	local function setupTooltip()
		local addonInfo = LE_SpecManager:AddonInfo(true);
		local activeSpecGroup = GetActiveSpecGroup();
		local currentSpecInfo = LE_SpecManager.currentSpecInfo;
		local lootSpecialization = GetLootSpecialization();
		local lootSpecInfo = LE_SpecManager.lootSpecInfo;

		iconTooltip:ClearLines();
		iconTooltip:AddLine(string.format("%s%s:%s %s", HIGHLIGHT_FONT_COLOR_CODE, addonInfo.title, "|r", addonInfo.version), 0.5, 0.8, 1);
		iconTooltip:AddLine(" ");
		
		for counter = 1, GetNumSpecGroups() do
			local specInfo = LE_SpecManager:GetSpecInfo(counter);

			iconTooltip:AddLine(string.format("%s%s:%s %s", HIGHLIGHT_FONT_COLOR_CODE, (specInfo and specInfo.name) or "Not Selected", "|r", (counter == activeSpecGroup and activeString) or inactiveString));
		end

		iconTooltip:AddLine(" ");
		iconTooltip:AddLine(string.format("%s%s:%s %s", HIGHLIGHT_FONT_COLOR_CODE, SELECT_LOOT_SPECIALIZATION, "|r", ((lootSpecialization == 0) and string.format(LOOT_SPECIALIZATION_DEFAULT, (lootSpecInfo and lootSpecInfo.name) or "None")) or lootSpecInfo.name), 0.5, 0.8, 1);

		if IsShiftKeyDown() then
			iconTooltip:AddLine(" ");
			iconTooltip:AddLine(string.join("", HIGHLIGHT_FONT_COLOR_CODE, "Mouse Actions using Shift Key", "|r"));		
			iconTooltip:AddLine(string.join(" ", "Toggle Minimap icon:", string.join("", "|cff00FF00", "Shift", " + ", "|r"), leftButtonTextureString), 0.5, 0.8, 1);		
			iconTooltip:AddLine(string.join(" ", "Reload UI:", string.join("", "|cff00FF00", "Shift", " + ", "|r"), rightButtonTextureString), 0.5, 0.8, 1);		
		else
			iconTooltip:AddLine(" ");
			iconTooltip:AddLine(string.join("", HIGHLIGHT_FONT_COLOR_CODE, "Mouse Actions", "|r"));		
			iconTooltip:AddLine(string.join(" ", "Change Talent Specialization:", leftButtonTextureString), 0.5, 0.8, 1);
			iconTooltip:AddLine(string.join(" ", "Change Loot Specialization:", rightButtonTextureString), 0.5, 0.8, 1);
		end

		iconTooltip:Show();
	end

	local function OnLeftClick()
		if IsShiftKeyDown() then
			LDBManager:ToggleMinimapIcon();
		else
			local numSpecGroups = GetNumSpecGroups();

			if numSpecGroups == 2 then
				SetActiveSpecGroup(GetActiveSpecGroup() == 1 and 2 or 1);
			end
		end
	end
	
	local function OnRightClick()
		if IsShiftKeyDown() then
			StaticPopup_Show("LESM_CONFIRM_RELOAD_UI");
		else
			local currentSpecInfo = LE_SpecManager.currentSpecInfo;

			iconTooltip:Hide()
			menuList[2].text = string.format(LOOT_SPECIALIZATION_DEFAULT, (currentSpecInfo and currentSpecInfo.name) or "None");

			for counter = 1, 4 do
				local specID, specName = GetSpecializationInfo(counter);

				if specID then
					menuList[counter + 2].text = specName;
					menuList[counter + 2].func = function() SetLootSpecialization(specID) end;
				else
					menuList[counter + 2] = nil;
				end
			end

			EasyMenu(menuList, menuFrame, "cursor", -15, -7, "MENU", 2);
		end
	end
	
	local function OnClick(frame, button)
		if button == "LeftButton" then
			OnLeftClick();
		elseif button == "RightButton" then
			OnRightClick();
		end
	end

	local function OnLeave(self)
		iconTooltip = nil;
	end

	local function OnEnter(tooltip)
		iconTooltip = tooltip;
		setupTooltip();
	end

	function LDBManager:OnInitialize()		
		local profile = LE_SpecManager.LDBProfile.profile;			
		local LDBIconData = {
			type = "data source",
			OnClick = OnClick,
			OnLeave = OnLeave,
			OnTooltipShow = OnEnter,
		};

		LDBIconData = LE_SpecManager:MergeTables(LDBIconData, LDBDefaults);
		LDBIconData = LE_SpecManager:MergeTables(LDBIconData, profile.LDBIconStorage);

		reloadRequired = profile.LDBIconStorage.hide;

		dataBroker = baseDataBroker:NewDataObject(dataTextName, LDBIconData);			
		addonIcon:Register(dataTextName, dataBroker, profile.LDBIconStorage);
	end
	
	function LDBManager:OnEnable()
		self:RegisterEvent("MODIFIER_STATE_CHANGED", "UpdateToolTip");

		self:RegisterMessage("LE_SpecManager_Load", "UpdateLDB");
		self:RegisterMessage("LE_SpecManager_SpecAdded", "UpdateLDB");
		self:RegisterMessage("LE_SpecManager_SpecChange_Finish", "UpdateLDB");
		self:RegisterMessage("LE_SpecManager_SpecRemoved", "UpdateLDB");
		self:RegisterMessage("LE_SpecManager_LootSpec_Changed", "UpdateLDB");
		self:RegisterMessage("LE_SpecManager_MinimapIcon_Toggle", "ToggleMinimapIcon");
	end

	function LDBManager:OnDisable()
		self:UnregisterMessage("LE_SpecManager_Load");
		self:UnregisterMessage("LE_SpecManager_SpecAdded");
		self:UnregisterMessage("LE_SpecManager_SpecChange_Finish");
		self:UnregisterMessage("LE_SpecManager_SpecRemoved");
		self:UnregisterMessage("LE_SpecManager_LootSpec_Changed");
		self:UnregisterMessage("LE_SpecManager_MinimapIcon_Toggle");

		self:UnregisterEvent("MODIFIER_STATE_CHANGED");
	end

	function LDBManager:UpdateToolTip()
		if iconTooltip then
			setupTooltip();
		end
	end

	function LDBManager:UpdateLDB()
		local profile = LE_SpecManager.LDBProfile.profile;

		local currentSpecInfo = LE_SpecManager.currentSpecInfo;
		local lootSpecInfo = LE_SpecManager.lootSpecInfo;
		local talentText = (currentSpecInfo and string.format('|T%s:14:14:0:0:64:64:4:60:4:60|t', currentSpecInfo.icon)) or "N/A";
		local lootText = (lootSpecInfo and string.format('|T%s:14:14:0:0:64:64:4:60:4:60|t', lootSpecInfo.icon)) or "N/A";

		dataBroker.text = string.format('%s: %s   %s: %s', "Spec", talentText, LOOT, lootText) or LDBDefaults.text;
		dataBroker.icon = (currentSpecInfo and currentSpecInfo.icon) or LDBDefaults.icon;
		
		for key, value in pairs(profile.LDBIconStorage) do
			dataBroker[key] = value;
		end

		addonIcon:Refresh(dataTextName, profile.LDBIconStorage);

		if iconTooltip then
			setupTooltip();
		end
	end

	function LDBManager:ToggleMinimapIcon()
		local profile = LE_SpecManager.LDBProfile.profile;

		profile.LDBIconStorage.hide = not profile.LDBIconStorage.hide;

		if profile.LDBIconStorage.hide then
			addonIcon:Hide(dataTextName);
		else
			addonIcon:Show(dataTextName);

			if reloadRequired then
				StaticPopup_Show("LESM_TOGGLE_MINIMAP");
				reloadRequired = false;
			end
		end
	end
end