local FORCE_DEBUG = false;
local Debug = LE_SpecManager.Debug;

if LE_SpecManager:IsCompatableWOWVersion() then
	local moduleName = "MacroManager";
	local MacroManager = LE_SpecManager:NewModule(moduleName, "AceConsole-3.0", "AceEvent-3.0");
	LE_SpecManager.MacroManager = MacroManager;

	local combatLockdown = false;

	local function Macro_ClearMacros()
		for counter = 138, 121, -1 do 
			DeleteMacro(counter);
		end
	end

	local function Macro_CreateMacros(specInfo)
		local profile = LE_SpecManager.macroProfile.profile[specInfo.name];

		for index, keys in pairs(profile) do   
			if profile[index] then
				local macroName = profile[index].Name;
				local macroBody = profile[index].Body;
				local macroTexture = profile[index].Texture;

				if string.find(macroBody, "mod:") 
					or string.find(macroBody, "modifier")
					or string.find(macroBody, "#showtooltip") then
					macroTexture = "INV_MISC_QUESTIONMARK";
				end

				if tonumber(index) >= 121 then
					CreateMacro(macroName, macroTexture, macroBody, 1);
				end
			end
		end
	end

	local function Macro_GetButtonIDs(macroID)
		local returnValue = {};

		for counter = 1, 120 do
			local actionType, actionID = GetActionInfo(counter);

			if (actionType and actionType == "macro") and (actionID and actionID == macroID) then
				table.insert(returnValue, counter);
			end
		end
		
		return returnValue;
	end

	local function Macro_GetMacro(macroID)
		local returnValue = nil;
		local macroButtonIDs = Macro_GetButtonIDs(macroID);
		local macroName, macroTexture, macroBody = GetMacroInfo(macroID);

		if macroName and macroBody and macroTexture then
			returnValue = {
				name = macroName,
				body = macroBody,
				texture = string.gsub(string.upper(macroTexture or "INV_Misc_Note_01"), "INTERFACE\\ICONS\\", ""),
				buttonIDs = macroButtonIDs,
			};
		end

		return returnValue;
	end

	local function ActionBar_MoveMacros(specInfo)
		local profile = LE_SpecManager.macroProfile.profile[specInfo.name];

		for index, keys in pairs(profile) do   
			if profile[index].ButtonIDs then	 
				for _, buttonID in pairs(profile[index].ButtonIDs) do
					ClearCursor();

					-- Clear current spot
					PickupAction(buttonID);
					ClearCursor();

					-- Restore this spot
					PickupMacro(tonumber(index));
					PlaceAction(buttonID);
					ClearCursor();
				end
			end
		end
	end
	
	local function MacroProfile_Exists(specInfo)
		return (specInfo and LE_SpecManager.macroProfile.profile[specInfo.name] ~= nil) or false;
	end

	function MacroManager:OnEnable()
		self:RegisterMessage("LE_SpecManager_Load", "Save");
		self:RegisterMessage("LE_SpecManager_SpecAdded", "Load");
		self:RegisterMessage("LE_SpecManager_SpecChange_Start", "Save");
		self:RegisterMessage("LE_SpecManager_SpecChange_Finish", "Load");
		self:RegisterMessage("LE_SpecManager_SpecRemoved", "Save");
		self:RegisterMessage("LE_SpecManager_LeftCombat", "Load");
	end

	function MacroManager:OnDisable()
		self:UnregisterMessage("LE_SpecManager_Load");
		self:UnregisterMessage("LE_SpecManager_SpecAdded");
		self:UnregisterMessage("LE_SpecManager_SpecChange_Start");
		self:UnregisterMessage("LE_SpecManager_SpecChange_Finish");
		self:UnregisterMessage("LE_SpecManager_SpecRemoved");
		self:UnregisterMessage("LE_SpecManager_LeftCombat");
	end

	function MacroManager:Load(event, specInfo)
		if specInfo and (event ~= "LE_SpecManager_LeftCombat" or (event == "LE_SpecManager_LeftCombat" and combatLockdown)) then
			if MacroProfile_Exists(specInfo) then
				combatLockdown = InCombatLockdown();

				if not combatLockdown then		
					Macro_ClearMacros();
					Macro_CreateMacros(specInfo);
					ActionBar_MoveMacros(specInfo);			

					if (IsAddOnLoaded("Blizzard_MacroUI")) then
						MacroFrame_Update();
					end
				end
			else
				MacroManager:Save(event, specInfo);
			end
		end
	end

	function MacroManager:Save(event, specInfo)
		if specInfo then
			LE_SpecManager.macroProfile.profile[specInfo.name] = {};

			local profile = LE_SpecManager.macroProfile.profile[specInfo.name];

			for counter = 121, 138 do 
				local macroInfo = Macro_GetMacro(counter);

				if macroInfo then
					profile[tostring(counter)] = { 
						Name = macroInfo.name,
						Body = macroInfo.body,
						Texture = macroInfo.texture,
						ButtonIDs = macroInfo.buttonIDs,
					};
				end
			end
		end
	end
end