local addonName = "LE_SpecManager";
local LE_SpecManager = LibStub("AceAddon-3.0"):NewAddon(addonName, "AceConsole-3.0", "AceEvent-3.0");
LE_SpecManager.addonName = addonName;
_G[addonName] = LE_SpecManager;

local FORCE_DEBUG = false;
local Debug = nil;

local WOW_VERSION_MIN = 60000;
local WOW_VERSION_MAX = 60200;

function LE_SpecManager:IsCompatableWOWVersion()
	if WOW_VERSION_MIN and WOW_VERSION_MAX then
		local versionRunning = select(4, GetBuildInfo());
		
		return (versionRunning >= WOW_VERSION_MIN) and (versionRunning <= WOW_VERSION_MAX);
	end
	
	return false;
end

if LE_SpecManager:IsCompatableWOWVersion() then
	local ADDON_NAME = "LunaEclipse_SpecManager";
	local ADDON_TITLE = GetAddOnMetadata(ADDON_NAME, "title");
	local ADDON_AUTHOR = GetAddOnMetadata(ADDON_NAME, "author");
	local ADDON_VERSION = GetAddOnMetadata(ADDON_NAME, "version");
	local majorVersion, minorVersion, revisionVersion, buildVersion = string.split(".", ADDON_VERSION);

	local currentSpecGroupID = nil;

	local ACTIVATE_PRIMARY_SPEC = 63645;
	local ACTIVATE_SECONDARY_SPEC = 63644;
	
	local LDBProfile_Defaults = {
		profile = {
			LDBIconStorage = {
				hide = false,
			},
		},
	};

	LE_SpecManager.currentSpecInfo = nil;
	LE_SpecManager.lootSpecInfo = nil;
	LE_SpecManager.previousSpecInfo = nil;
	LE_SpecManager.versionInfo = {
		major = majorVersion,
		minor = minorVersion,
		revision = revisionVersion,
		build = buildVersion,
	};

	function LE_SpecManager:CMDLine(...)
		self:SendMessage(string.join("_", addonName, "MinimapIcon", "Toggle"));
	end

	function LE_SpecManager:AddonInfo(stripTags)
		local returnValue = {
			title = (stripTags and ADDON_TITLE) or string.join("", "|cff1784d1", ADDON_TITLE, "|r"),
			author = (stripTags and ADDON_AUTHOR) or string.join("", "|cffffd700", ADDON_AUTHOR, "|r"),
			version = (stripTags and ADDON_VERSION) or string.join("", "|cff1784d1", ADDON_VERSION, "|r"),
		};

		return returnValue;
	end
	
	function LE_SpecManager:GetLootSpecInfo(specInfoID)
		local returnValue = nil;		
		local lootSpecialization = GetLootSpecialization();
		local specIndex = specInfoID or (((lootSpecialization == 0 and self.currentSpecInfo) and self.currentSpecInfo.id) or (lootSpecialization ~= 0 and lootSpecialization));
		
		if specIndex then 
			local specID, specName, specDescription, specIcon, specBackground, specRole, specClass = GetSpecializationInfoByID(specIndex);

			returnValue = {
				id = specID,
				name = specName,
				description = specDescription,
				icon = specIcon,
				background = specBackground,
				role = specRole,
				class = specClass,
			};
		end
		
		return returnValue;
	end

	function LE_SpecManager:GetSpecInfo(specInfoID)
		local returnValue = nil;		
		local specIndex = (specInfoID and GetSpecialization(false, false, specInfoID)) or GetSpecialization();

		if specIndex then 
			local specID, specName, specDescription, specIcon, specBackground, specRole, specPrimaryStat = GetSpecializationInfo(specIndex);

			returnValue = {
				id = specID,
				name = specName,
				description = specDescription,
				icon = specIcon,
				background = specBackground,
				role = specRole,
				primaryStat = specPrimaryStat,
			};
		end
		
		return returnValue;
	end

	function LE_SpecManager:DuplicateTables(originalTable)
		local returnValue = {};

		if originalTable then
			for key, value in pairs(originalTable) do
				if type(value) == "table" then
					returnValue[key] = self:DuplicateTables(value);
				else
					returnValue[key] = value;
				end
			end
		end
		
		return returnValue;
	end

	function LE_SpecManager:MergeTables(returnValue, mergeTable)
		if mergeTable then
			for key, value in pairs(mergeTable) do
				returnValue[key] = value;
			end
		end
		
		return returnValue;
	end

	function LE_SpecManager:PLAYER_LOGIN(event, ...)
		self.currentSpecInfo = self:GetSpecInfo();
		self.lootSpecInfo = self:GetLootSpecInfo();
		currentSpecGroupID = GetActiveSpecGroup();
		self:SendMessage(string.join("_", addonName, "Load"), self.currentSpecInfo);
	end

	function LE_SpecManager:UNIT_SPELLCAST_START(event, ...)
		local eventUnit, _, _, _, spellID = select(1, ...);
		
		if eventUnit == "player" and (spellID == ACTIVATE_PRIMARY_SPEC or spellID == ACTIVATE_SECONDARY_SPEC) then
			self:SendMessage(string.join("_", addonName, "SpecChange", "Start"), self.currentSpecInfo);
		end
	end

	function LE_SpecManager:PLAYER_SPECIALIZATION_CHANGED(event, ...)
		local eventUnit = select(1, ...);

		if eventUnit == "player" then
			local currentSpecInfo = self:DuplicateTables(self.currentSpecInfo);
			local previousSpecGroupID = currentSpecGroupID;
			
			currentSpecGroupID = GetActiveSpecGroup();
			self.currentSpecInfo = self:GetSpecInfo();
			self.lootSpecInfo = self:GetLootSpecInfo();
	
			if currentSpecGroupID == previousSpecGroupID then
				if self.currentSpecInfo and not currentSpecInfo then
					self.previousSpecInfo = nil;
					self:SendMessage(string.join("_", addonName, "SpecAdded"), self.currentSpecInfo);
				elseif currentSpecInfo and not self.currentSpecInfo then
					self.previousSpecInfo = self:DuplicateTables(currentSpecInfo);
					self:SendMessage(string.join("_", addonName, "SpecRemoved"), self.previousSpecInfo);
				end
			else
				if self.currentSpecInfo ~= currentSpecInfo then
					self.previousSpecInfo = self:DuplicateTables(currentSpecInfo);
					self:SendMessage(string.join("_", addonName, "SpecChange", "Finish"), self.currentSpecInfo);
				end
			end
		end
	end
	
	function LE_SpecManager:PLAYER_LOOT_SPEC_UPDATED(event, ...)
		self.lootSpecInfo = self:GetLootSpecInfo();
		self:SendMessage(string.join("_", addonName, "LootSpec", "Changed"), self.lootSpecInfo);
	end

	function LE_SpecManager:PLAYER_REGEN_ENABLED(event, ...)
		self:SendMessage(string.join("_", addonName, "LeftCombat"), self.currentSpecInfo);
	end

	function LE_SpecManager:PLAYER_LOGOUT(event, ...)
		self:SendMessage(string.join("_", addonName, "Close"), self.currentSpecInfo);
	end

	function LE_SpecManager:OnInitialize()		
		self.LDBProfile = LibStub("AceDB-3.0"):New("LDBProfile", LDBProfile_Defaults, "Settings");
		self.macroProfile = LibStub("AceDB-3.0"):New("MacroProfile", nil, "Settings");

		Debug = self.Debug;
		
		local addonInfo = self:AddonInfo();
		print(string.format("%s %s %s %s", addonInfo.title, "version:", addonInfo.version, "loaded!"));
	end

	function LE_SpecManager:OnEnable()
		self:RegisterEvent("PLAYER_LOGIN");
		self:RegisterEvent("UNIT_SPELLCAST_START");
		self:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED");
		self:RegisterEvent("PLAYER_LOOT_SPEC_UPDATED");
		self:RegisterEvent("PLAYER_REGEN_ENABLED");
		self:RegisterEvent("PLAYER_LOGOUT");
		
		self:RegisterChatCommand("LESM", "CMDLine");
	end

	function LE_SpecManager:OnDisable()
		self:UnregisterChatCommand("LESM");

		self:UnregisterEvent("PLAYER_LOGIN");
		self:UnregisterEvent("UNIT_SPELLCAST_START");
		self:UnregisterEvent("PLAYER_SPECIALIZATION_CHANGED");
		self:UnregisterEvent("PLAYER_LOOT_SPEC_UPDATED");
		self:UnregisterEvent("PLAYER_REGEN_ENABLED");
		self:UnregisterEvent("PLAYER_LOGOUT");
	end
end